@@ -33,16 +33,16 @@ typedef struct tagEXPECTEDNOTIFY
 
 typedef LRESULT (*CUSTOMDRAWPROC)(int n, NMCUSTOMDRAW *nm);
 
-CUSTOMDRAWPROC g_CustomDrawProc;
-int g_CustomDrawCount;
-DRAWITEMSTRUCT g_DrawItem;
-BOOL g_DrawItemReceived;
+static CUSTOMDRAWPROC g_CustomDrawProc;
+static int g_CustomDrawCount;
+static DRAWITEMSTRUCT g_DrawItem;
+static BOOL g_DrawItemReceived;
 
-EXPECTEDNOTIFY expectedNotify[10];
-INT nExpectedNotify = 0;
-INT nReceivedNotify = 0;
-INT unexpectedNotify[10];
-INT nUnexpectedNotify = 0;
+static EXPECTEDNOTIFY expectedNotify[10];
+static INT nExpectedNotify = 0;
+static INT nReceivedNotify = 0;
+static INT unexpectedNotify[10];
+static INT nUnexpectedNotify = 0;
 
 static HWND hHeaderParentWnd;
 static HWND hWndHeader;
@@ -73,22 +73,22 @@ static BOOL notifies_received(void)
     return fRet;
 }
 
-static LONG addItem(HWND hdex, int idx, LPCSTR text)
+static LONG addItem(HWND hdex, int idx, LPSTR text)
 {
     HDITEMA hdItem;
     hdItem.mask       = HDI_TEXT | HDI_WIDTH;
     hdItem.cxy        = 100;
-    hdItem.pszText    = (LPSTR)text;
+    hdItem.pszText    = text;
     hdItem.cchTextMax = 0;
     return (LONG)SendMessage(hdex, HDM_INSERTITEMA, (WPARAM)idx, (LPARAM)&hdItem);
 }
 
-static LONG setItem(HWND hdex, int idx, LPCSTR text, BOOL fCheckNotifies)
+static LONG setItem(HWND hdex, int idx, LPSTR text, BOOL fCheckNotifies)
 {
     LONG ret;
     HDITEMA hdexItem;
     hdexItem.mask       = HDI_TEXT;
-    hdexItem.pszText    = (LPSTR)text;
+    hdexItem.pszText    = text;
     hdexItem.cchTextMax = 0;
     if (fCheckNotifies)
     {
@@ -101,17 +101,17 @@ static LONG setItem(HWND hdex, int idx, LPCSTR text, BOOL fCheckNotifies)
     return ret;
 }
 
-static LONG setItemUnicodeNotify(HWND hdex, int idx, LPCSTR text, LPCWSTR wText)
+static LONG setItemUnicodeNotify(HWND hdex, int idx, LPSTR text, LPWSTR wText)
 {
     LONG ret;
     HDITEMA hdexItem;
     HDITEMW hdexNotify;
     hdexItem.mask       = HDI_TEXT;
-    hdexItem.pszText    = (LPSTR)text;
+    hdexItem.pszText    = text;
     hdexItem.cchTextMax = 0;
     
     hdexNotify.mask    = HDI_TEXT;
-    hdexNotify.pszText = (LPWSTR)wText;
+    hdexNotify.pszText = wText;
     
     expect_notify(HDN_ITEMCHANGINGW, TRUE, (HDITEMA*)&hdexNotify);
     expect_notify(HDN_ITEMCHANGEDW, TRUE, (HDITEMA*)&hdexNotify);
@@ -145,7 +145,7 @@ static void addReadDelItem(HWND hdex, HDITEMA *phdiCreate, int maskRead, HDITEMA
     ZeroMemory(phdiRead, sizeof(HDITEMA));
     phdiRead->mask = maskRead;
     ok(SendMessage(hdex, HDM_GETITEMA, (WPARAM)0, (LPARAM)phdiRead)!=0, "Getting item data failed\n");
-    ok(SendMessage(hdex, HDM_DELETEITEM, (WPARAM)0, (LPARAM)0)!=0, "Deleteing item failed\n");
+    ok(SendMessage(hdex, HDM_DELETEITEM, (WPARAM)0, (LPARAM)0)!=0, "Deleting item failed\n");
 }
 
 static HWND create_header_control (void)
@@ -207,11 +207,18 @@ static void compare_items(INT iCode, HDITEMA *hdi1, HDITEMA *hdi2, BOOL fUnicode
     }
 }
 
-static const char *str_items[] =
-    {"First Item", "Second Item", "Third Item", "Fourth Item", "Replace Item", "Out Of Range Item"};
+static char pszFirstItem[]      = "First Item";
+static char pszSecondItem[]     = "Second Item";
+static char pszThirdItem[]      = "Third Item";
+static char pszFourthItem[]     = "Fourth Item";
+static char pszReplaceItem[]    = "Replace Item";
+static char pszOutOfRangeItem[] = "Out Of Range Item";
+
+static char *str_items[] =
+    {pszFirstItem, pszSecondItem, pszThirdItem, pszFourthItem, pszReplaceItem, pszOutOfRangeItem};
     
-static const char pszUniTestA[] = "TST";
-static const WCHAR pszUniTestW[] = {'T','S','T',0};
+static char pszUniTestA[]  = "TST";
+static WCHAR pszUniTestW[] = {'T','S','T',0};
 
 
 #define TEST_GET_ITEM(i,c)\
@@ -611,7 +618,7 @@ static void run_customdraw_scenario(CUSTOMDRAWPROC proc)
     g_CustomDrawProc = NULL;
 }
 
-void test_customdraw()
+static void test_customdraw(void)
 {
     int i;
     HDITEM item;
@@ -735,7 +742,7 @@ static void test_header_order (void)
     DestroyWindow(hWndHeader);
 }
 
-LRESULT CALLBACK HeaderTestWndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
+static LRESULT CALLBACK HeaderTestWndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
 {
     DRAWITEMSTRUCT *di;
     switch(msg) {
