@@ -47,8 +47,8 @@ static const WCHAR chooseFontW[] = {'C','H','O','O','S','E','_','F','O','N','T',
 static HIMAGELIST himlTT = 0;
 #define TTBITMAP_XSIZE 20 /* x-size of the bitmaps */
 
-INT_PTR CALLBACK FormatCharDlgProcA(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
-INT_PTR CALLBACK FormatCharDlgProcW(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
+static INT_PTR CALLBACK FormatCharDlgProcA(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
+static INT_PTR CALLBACK FormatCharDlgProcW(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
 
 /* There is a table here of all charsets, and the sample text for each.
  * There is a second table that translates a charset into an index into
@@ -88,7 +88,7 @@ static const WCHAR stIso88594[]={'A','a','B','b',0}; /* ISO-8859-4 */
 static const WCHAR stIso885910[]={'A','a','B','b',0}; /* ISO-8859-10 */
 static const WCHAR stCeltic[]={'A','a','B','b',0};/* Celtic */
 
-static const WCHAR *sample_lang_text[]={
+static const WCHAR * const sample_lang_text[]={
     stWestern,stSymbol,stShiftJis,stHangul,stGB2312,
     stBIG5,stGreek,stTurkish,stHebrew,stArabic,
     stBaltic,stVietname,stCyrillic,stEastEur,stThai,
