@@ -60,6 +60,7 @@ static InputContextData *root_context = NULL;
 static HWND hwndDefault = NULL;
 static HANDLE hImeInst;
 static const WCHAR WC_IMECLASSNAME[] = {'I','M','E',0};
+static ATOM atIMEClass = 0;
 
 /* MSIME messages */
 static UINT WM_MSIME_SERVICE;
@@ -109,12 +110,14 @@ static void IMM_Register(void)
     wndClass.hbrBackground = (HBRUSH)(COLOR_WINDOW +1);
     wndClass.lpszMenuName   = 0;
     wndClass.lpszClassName = WC_IMECLASSNAME;
-    RegisterClassW(&wndClass);
+    atIMEClass = RegisterClassW(&wndClass);
 }
 
 static void IMM_Unregister(void)
 {
-    UnregisterClassW(WC_IMECLASSNAME, NULL);
+    if (atIMEClass) {
+        UnregisterClassW(WC_IMECLASSNAME, NULL);
+    }
 }
 
 static void IMM_RegisterMessages(void)
@@ -1566,7 +1569,7 @@ static LRESULT WINAPI IME_WindowProc(HWND hwnd, UINT msg, WPARAM wParam,
             TRACE("IME message %s, 0x%x, 0x%x (%i)\n",
                     "WM_IME_COMPOSITION", (UINT)wParam, (UINT)lParam,
                      root_context->bRead);
-            if ((lParam & GCS_RESULTSTR) && (!root_context->bRead))
+            if (lParam & GCS_RESULTSTR)
                     IMM_PostResult(root_context);
             else
                  UpdateDataInDefaultIMEWindow(hwnd);
