@@ -1281,6 +1281,7 @@ HANDLE WINAPI CreateFileW( LPCWSTR filename, DWORD access, DWORD sharing,
     static const WCHAR bkslashes_with_dotW[] = {'\\','\\','.','\\',0};
     static const WCHAR coninW[] = {'C','O','N','I','N','$',0};
     static const WCHAR conoutW[] = {'C','O','N','O','U','T','$',0};
+    SECURITY_QUALITY_OF_SERVICE qos;
 
     static const UINT nt_disposition[5] =
     {
@@ -1406,7 +1407,16 @@ HANDLE WINAPI CreateFileW( LPCWSTR filename, DWORD access, DWORD sharing,
     attr.Attributes = OBJ_CASE_INSENSITIVE;
     attr.ObjectName = &nameW;
     attr.SecurityDescriptor = sa ? sa->lpSecurityDescriptor : NULL;
-    attr.SecurityQualityOfService = NULL;
+    if (attributes & SECURITY_SQOS_PRESENT)
+    {
+        qos.Length = sizeof(qos);
+        qos.ImpersonationLevel = (attributes >> 16) & 0x3;
+        qos.ContextTrackingMode = attributes & SECURITY_CONTEXT_TRACKING ? SECURITY_DYNAMIC_TRACKING : SECURITY_STATIC_TRACKING;
+        qos.EffectiveOnly = attributes & SECURITY_EFFECTIVE_ONLY ? TRUE : FALSE;
+        attr.SecurityQualityOfService = &qos;
+    }
+    else
+        attr.SecurityQualityOfService = NULL;
 
     if (sa && sa->bInheritHandle) attr.Attributes |= OBJ_INHERIT;
 
@@ -1677,20 +1687,34 @@ HANDLE WINAPI FindFirstFileExW( LPCWSTR filename, FINDEX_INFO_LEVELS level,
         CloseHandle( info->handle );
         info->handle = 0;
     }
-    else if (!FindNextFileW( (HANDLE)info, data ))
+    else
     {
-        TRACE( "%s not found\n", debugstr_w(filename) );
-        FindClose( (HANDLE)info );
-        SetLastError( ERROR_FILE_NOT_FOUND );
-        return INVALID_HANDLE_VALUE;
+        IO_STATUS_BLOCK io;
+
+        NtQueryDirectoryFile( info->handle, 0, NULL, NULL, &io, info->data, sizeof(info->data),
+                              FileBothDirectoryInformation, FALSE, &info->mask, TRUE );
+        if (io.u.Status)
+        {
+            FindClose( info );
+            SetLastError( RtlNtStatusToDosError( io.u.Status ) );
+            return INVALID_HANDLE_VALUE;
+        }
+        info->data_len = io.Information;
+        if (!FindNextFileW( info, data ))
+        {
+            TRACE( "%s not found\n", debugstr_w(filename) );
+            FindClose( info );
+            SetLastError( ERROR_FILE_NOT_FOUND );
+            return INVALID_HANDLE_VALUE;
+        }
+        if (!strpbrkW( info->mask.Buffer, wildcardsW ))
+        {
+            /* we can't find two files with the same name */
+            CloseHandle( info->handle );
+            info->handle = 0;
+        }
     }
-    else if (!strpbrkW( info->mask.Buffer, wildcardsW ))
-    {
-        /* we can't find two files with the same name */
-        CloseHandle( info->handle );
-        info->handle = 0;
-    }
-    return (HANDLE)info;
+    return info;
 
 error:
     HeapFree( GetProcessHeap(), 0, info );
