@@ -824,8 +824,8 @@ MMRESULT MSACM_Message(HACMDRIVER had, UINT uMsg, LPARAM lParam1, LPARAM lParam2
     return MMSYSERR_INVALHANDLE;
 }
 
-PWINE_ACMLOCALDRIVER MSACM_pFirstACMLocalDriver = NULL;
-PWINE_ACMLOCALDRIVER MSACM_pLastACMLocalDriver = NULL;
+static PWINE_ACMLOCALDRIVER MSACM_pFirstACMLocalDriver;
+static PWINE_ACMLOCALDRIVER MSACM_pLastACMLocalDriver;
 
 PWINE_ACMLOCALDRIVER MSACM_RegisterLocalDriver(HMODULE hModule, DRIVERPROC lpDriverProc)
 {
