@@ -981,17 +981,6 @@ static	LRESULT	G711_StreamClose(PACMDRVSTREAMINSTANCE adsi)
     return MMSYSERR_NOERROR;
 }
 
-/***********************************************************************
- *           G711_round
- *
- */
-static	inline DWORD	G711_round(DWORD a, DWORD b, DWORD c)
-{
-    assert(a && b && c);
-    /* to be sure, always return an entire number of c... */
-    return ((double)a * (double)b + (double)c - 1) / (double)c;
-}
-
 /***********************************************************************
  *           G711_StreamSize
  *
