@@ -38,12 +38,12 @@ static HINSTANCE MSRLE32_hModule = 0;
 #define SQR(a)                ((a) * (a))
 
 #define QUALITY_to_DIST(q)    (ICQUALITY_HIGH - q)
-inline WORD ColorCmp(WORD clr1, WORD clr2)
+static inline WORD ColorCmp(WORD clr1, WORD clr2)
 {
   register UINT a = (clr1-clr2);
   return SQR(a);
 }
-inline WORD Intensity(RGBQUAD clr)
+static inline WORD Intensity(RGBQUAD clr)
 {
   return (30 * clr.rgbRed + 59 * clr.rgbGreen + 11 * clr.rgbBlue)/4;
 }
@@ -101,12 +101,12 @@ static BOOL isSupportedMRLE(LPCBITMAPINFOHEADER lpbi)
   /* pre-conditions */
   assert(lpbi != NULL);
 
-  if (lpbi->biSize < sizeof(BITMAPINFOHEADER) || \
+  if (lpbi->biSize < sizeof(BITMAPINFOHEADER) ||
       lpbi->biPlanes != 1)
     return FALSE;
 
   if (lpbi->biCompression == BI_RLE4) {
-    if (lpbi->biBitCount != 4 || \
+    if (lpbi->biBitCount != 4 ||
 	(lpbi->biWidth % 2) != 0)
       return FALSE;
   } else if (lpbi->biCompression == BI_RLE8) {
