@@ -68,7 +68,7 @@ typedef struct _MSVCRT_EXCEPTION_FRAME
 #if defined(__GNUC__) && defined(__i386__)
 inline static void call_finally_block( void *code_block, void *base_ptr )
 {
-    __asm__ __volatile__ ("movl %1,%%ebp; call *%%eax" \
+    __asm__ __volatile__ ("movl %1,%%ebp; call *%%eax"
                           : : "a" (code_block), "g" (base_ptr));
 }
 
@@ -109,7 +109,7 @@ __ASM_GLOBAL_FUNC(_EH_prolog,
                   "movl  %ebp, 12(%esp)\n\t"
                   "leal  12(%esp), %ebp\n\t"
                   "pushl %eax\n\t"
-                  "ret");
+                  "ret")
 #endif
 
 /*******************************************************************
@@ -259,7 +259,7 @@ int CDECL _abnormal_termination(void)
 
 #ifdef __i386__
 #define MSVCRT_JMP_MAGIC 0x56433230 /* ID value for new jump structure */
-typedef void (*MSVCRT_unwind_function)(const void*);
+typedef void (__stdcall *MSVCRT_unwind_function)(const struct MSVCRT___JUMP_BUFFER *);
 
 /* define an entrypoint for setjmp/setjmp3 that stores the registers in the jmp buf */
 /* and then jumps to the C backend function */
@@ -286,7 +286,7 @@ __ASM_GLOBAL_FUNC( longjmp_set_regs,
                    "movl 12(%ecx),%esi\n\t"  /* jmp_buf.Esi */
                    "movl 16(%ecx),%esp\n\t"  /* jmp_buf.Esp */
                    "addl $4,%esp\n\t"        /* get rid of return address */
-                   "jmp *20(%ecx)\n\t"       /* jmp_buf.Eip */ );
+                   "jmp *20(%ecx)\n\t"       /* jmp_buf.Eip */ )
 
 /*
  * The signatures of the setjmp/longjmp functions do not match that
@@ -297,7 +297,7 @@ __ASM_GLOBAL_FUNC( longjmp_set_regs,
 /*******************************************************************
  *		_setjmp (MSVCRT.@)
  */
-DEFINE_SETJMP_ENTRYPOINT(MSVCRT__setjmp);
+DEFINE_SETJMP_ENTRYPOINT(MSVCRT__setjmp)
 int CDECL __regs_MSVCRT__setjmp(struct MSVCRT___JUMP_BUFFER *jmp)
 {
     jmp->Registration = (unsigned long)NtCurrentTeb()->Tib.ExceptionList;
@@ -314,7 +314,7 @@ int CDECL __regs_MSVCRT__setjmp(struct MSVCRT___JUMP_BUFFER *jmp)
 /*******************************************************************
  *		_setjmp3 (MSVCRT.@)
  */
-DEFINE_SETJMP_ENTRYPOINT( MSVCRT__setjmp3 );
+DEFINE_SETJMP_ENTRYPOINT( MSVCRT__setjmp3 )
 int CDECL __regs_MSVCRT__setjmp3(struct MSVCRT___JUMP_BUFFER *jmp, int nb_args, ...)
 {
     jmp->Cookie = MSVCRT_JMP_MAGIC;
@@ -412,7 +412,7 @@ static BOOL WINAPI msvcrt_console_handler(DWORD ctrlType)
 typedef void (*float_handler)(int, int);
 
 /* The exception codes are actually NTSTATUS values */
-struct
+static const struct
 {
     NTSTATUS status;
     int signal;
