@@ -130,7 +130,7 @@ __ASM_GLOBAL_FUNC(vm86_enter,
                   ".globl " __ASM_NAME("vm86_return_end") "\n\t"
                   __ASM_FUNC("vm86_return_end") "\n"
                   __ASM_NAME("vm86_return_end") ":\n\t"
-                  "ret" );
+                  "ret" )
 
 #ifdef HAVE_SYS_VM86_H
 # define __HAVE_VM86
@@ -1019,11 +1019,15 @@ static void WINAPI raise_trap_exception( EXCEPTION_RECORD *rec, CONTEXT *context
         {
             context->ContextFlags = CONTEXT_DEBUG_REGISTERS;
             NtGetContextThread(GetCurrentThread(), context);
-            /* do we really have a bp from a debug register ?
-             * if not, then someone did a kill(SIGTRAP) on us, and we
-             * shall return a breakpoint, not a single step exception
+            /* we have either:
+             *    - a bp from a debug register
+             *    - a single step interrupt at popf instruction, which just has
+             *      removed the TF.
+             *    - someone did a kill(SIGTRAP) on us, and we shall return
+             *      a breakpoint, not a single step exception
              */
-            if (!(context->Dr6 & 0xf)) rec->ExceptionCode = EXCEPTION_BREAKPOINT;
+            if ( !(context->Dr6 & 0xf) && !(context->Dr6 & 0x4000) )
+                rec->ExceptionCode = EXCEPTION_BREAKPOINT;
             context->ContextFlags |= CONTEXT_FULL;  /* restore flags */
         }
     }
@@ -1337,12 +1341,8 @@ BOOL SIGNAL_Init(void)
     }
 #endif  /* HAVE_SIGALTSTACK */
 
-    sigemptyset( &sig_act.sa_mask );
-    sigaddset( &sig_act.sa_mask, SIGINT );
-    sigaddset( &sig_act.sa_mask, SIGUSR1 );
-    sigaddset( &sig_act.sa_mask, SIGUSR2 );
+    sig_act.sa_mask = server_block_set;
     sig_act.sa_flags = SA_SIGINFO | SA_RESTART;
-
 #ifdef SA_ONSTACK
     sig_act.sa_flags |= SA_ONSTACK;
 #endif
@@ -1473,12 +1473,12 @@ void __wine_enter_vm86( CONTEXT *context )
 /**********************************************************************
  *		DbgBreakPoint   (NTDLL.@)
  */
-__ASM_GLOBAL_FUNC( DbgBreakPoint, "int $3; ret");
+__ASM_GLOBAL_FUNC( DbgBreakPoint, "int $3; ret")
 
 /**********************************************************************
  *		DbgUserBreakPoint   (NTDLL.@)
  */
-__ASM_GLOBAL_FUNC( DbgUserBreakPoint, "int $3; ret");
+__ASM_GLOBAL_FUNC( DbgUserBreakPoint, "int $3; ret")
 
 
 /**********************************************************************
@@ -1505,7 +1505,7 @@ __ASM_GLOBAL_FUNC( EXC_CallHandler,
 "	popl	%ebx\n"
 "	leave\n"
 "	ret\n"
-);
+)
 __ASM_GLOBAL_FUNC(call_exception_handler,
 "	pushl	%ebp\n"
 "	movl	%esp, %ebp\n"
@@ -1529,5 +1529,5 @@ __ASM_GLOBAL_FUNC(call_exception_handler,
 "	movl	%ebp, %esp\n"     /* restore saved stack, in case it was corrupted */
 "	popl	%ebp\n"
 "	ret	$20\n"            /* (*4) */
-);
+)
 #endif  /* __i386__ */
