@@ -320,6 +320,9 @@ static const struct tagTZ_INFO TZ_INFO[] =
    {"WST",
     {'W','.',' ','A','u','s','t','r','a','l','i','a',' ','S','t','a','n','d',
      'a','r','d',' ','T','i','m','e','\0'}, -480, 0},
+   {"WST",
+    {'W','.',' ','A','u','s','t','r','a','l','i','a',' ','S','u','m','m','e',
+     'r',' ','T','i','m','e','\0'}, -540, 1},
    {"JST",
     {'T','o','k','y','o',' ','S','t','a','n','d','a','r','d',' ','T','i','m',
      'e','\0'}, -540, 0},
