@@ -812,7 +812,7 @@ LONG_PTR WINAPIV NdrClientCall2(PMIDL_STUB_DESC pStubDesc, PFORMAT_STRING pForma
     if (pProcHeader->Oi_flags & RPC_FC_PROC_OIF_OBJECT)
     {
         /* object is always the first argument */
-        This = **(void ***)(&pFormat+1);
+        This = **(void *const **)(&pFormat+1);
         NdrProxyInitialize(This, &rpcMsg, &stubMsg, pStubDesc, procedure_number);
     }
     else
@@ -1024,7 +1024,7 @@ __ASM_GLOBAL_FUNC(call_server_func,
     "pushl %esi\n\t"
     "movl 16(%ebp), %eax\n\t"   /* Get stack size */
     "subl %eax, %esp\n\t"       /* Make room in stack for arguments */
-    "andl $~15, %esp\n\t"	/* Make sure stack has 16-byte alignment for MacOS X */
+    "andl $~15, %esp\n\t"	/* Make sure stack has 16-byte alignment for Mac OS X */
     "movl %esp, %edi\n\t"
     "movl %eax, %ecx\n\t"
     "movl 12(%ebp), %esi\n\t"
@@ -1036,7 +1036,7 @@ __ASM_GLOBAL_FUNC(call_server_func,
     "popl %esi\n\t"             /* Restore registers */
     "popl %edi\n\t"
     "popl %ebp\n\t"
-    "ret\n" );
+    "ret\n" )
 #else
 #warning call_server_func not implemented for your architecture
 LONG_PTR __cdecl call_server_func(SERVER_ROUTINE func, unsigned char * args, unsigned short stack_size)
