@@ -57,7 +57,7 @@ LPSTR RPCRT4_strndupA(LPCSTR src, INT slen)
   return s;
 }
 
-LPSTR RPCRT4_strdupWtoA(LPWSTR src)
+LPSTR RPCRT4_strdupWtoA(LPCWSTR src)
 {
   DWORD len;
   LPSTR s;
@@ -68,7 +68,7 @@ LPSTR RPCRT4_strdupWtoA(LPWSTR src)
   return s;
 }
 
-LPWSTR RPCRT4_strdupAtoW(LPSTR src)
+LPWSTR RPCRT4_strdupAtoW(LPCSTR src)
 {
   DWORD len;
   LPWSTR s;
@@ -79,7 +79,7 @@ LPWSTR RPCRT4_strdupAtoW(LPSTR src)
   return s;
 }
 
-LPWSTR RPCRT4_strndupW(LPWSTR src, INT slen)
+LPWSTR RPCRT4_strndupW(LPCWSTR src, INT slen)
 {
   DWORD len;
   LPWSTR s;
@@ -110,7 +110,7 @@ static RPC_STATUS RPCRT4_AllocBinding(RpcBinding** Binding, BOOL server)
   return RPC_S_OK;
 }
 
-RPC_STATUS RPCRT4_CreateBindingA(RpcBinding** Binding, BOOL server, LPSTR Protseq)
+static RPC_STATUS RPCRT4_CreateBindingA(RpcBinding** Binding, BOOL server, LPSTR Protseq)
 {
   RpcBinding* NewBinding;
 
@@ -123,7 +123,7 @@ RPC_STATUS RPCRT4_CreateBindingA(RpcBinding** Binding, BOOL server, LPSTR Protse
   return RPC_S_OK;
 }
 
-RPC_STATUS RPCRT4_CreateBindingW(RpcBinding** Binding, BOOL server, LPWSTR Protseq)
+static RPC_STATUS RPCRT4_CreateBindingW(RpcBinding** Binding, BOOL server, LPWSTR Protseq)
 {
   RpcBinding* NewBinding;
 
@@ -136,7 +136,8 @@ RPC_STATUS RPCRT4_CreateBindingW(RpcBinding** Binding, BOOL server, LPWSTR Prots
   return RPC_S_OK;
 }
 
-RPC_STATUS RPCRT4_CompleteBindingA(RpcBinding* Binding, LPSTR NetworkAddr,  LPSTR Endpoint,  LPSTR NetworkOptions)
+static RPC_STATUS RPCRT4_CompleteBindingA(RpcBinding* Binding, LPSTR NetworkAddr,
+                                          LPSTR Endpoint, LPSTR NetworkOptions)
 {
   TRACE("(RpcBinding == ^%p, NetworkAddr == %s, EndPoint == %s, NetworkOptions == %s)\n", Binding,
    debugstr_a(NetworkAddr), debugstr_a(Endpoint), debugstr_a(NetworkOptions));
@@ -149,12 +150,15 @@ RPC_STATUS RPCRT4_CompleteBindingA(RpcBinding* Binding, LPSTR NetworkAddr,  LPST
   } else {
     Binding->Endpoint = RPCRT4_strdupA("");
   }
+  HeapFree(GetProcessHeap(), 0, Binding->NetworkOptions);
+  Binding->NetworkOptions = RPCRT4_strdupAtoW(NetworkOptions);
   if (!Binding->Endpoint) ERR("out of memory?\n");
 
   return RPC_S_OK;
 }
 
-RPC_STATUS RPCRT4_CompleteBindingW(RpcBinding* Binding, LPWSTR NetworkAddr, LPWSTR Endpoint, LPWSTR NetworkOptions)
+static RPC_STATUS RPCRT4_CompleteBindingW(RpcBinding* Binding, LPWSTR NetworkAddr,
+                                          LPWSTR Endpoint, LPWSTR NetworkOptions)
 {
   TRACE("(RpcBinding == ^%p, NetworkAddr == %s, EndPoint == %s, NetworkOptions == %s)\n", Binding, 
    debugstr_w(NetworkAddr), debugstr_w(Endpoint), debugstr_w(NetworkOptions));
@@ -168,6 +172,8 @@ RPC_STATUS RPCRT4_CompleteBindingW(RpcBinding* Binding, LPWSTR NetworkAddr, LPWS
     Binding->Endpoint = RPCRT4_strdupA("");
   }
   if (!Binding->Endpoint) ERR("out of memory?\n");
+  HeapFree(GetProcessHeap(), 0, Binding->NetworkOptions);
+  Binding->NetworkOptions = RPCRT4_strdupW(NetworkOptions);
 
   return RPC_S_OK;
 }
@@ -224,6 +230,9 @@ RPC_STATUS RPCRT4_DestroyBinding(RpcBinding* Binding)
   RPCRT4_strfree(Binding->Endpoint);
   RPCRT4_strfree(Binding->NetworkAddr);
   RPCRT4_strfree(Binding->Protseq);
+  HeapFree(GetProcessHeap(), 0, Binding->NetworkOptions);
+  if (Binding->AuthInfo) RpcAuthInfo_Release(Binding->AuthInfo);
+  if (Binding->QOS) RpcQualityOfService_Release(Binding->QOS);
   HeapFree(GetProcessHeap(), 0, Binding);
   return RPC_S_OK;
 }
@@ -241,7 +250,7 @@ RPC_STATUS RPCRT4_OpenBinding(RpcBinding* Binding, RpcConnection** Connection,
     /* try to find a compatible connection from the connection pool */
     NewConnection = RPCRT4_GetIdleConnection(InterfaceId, TransferSyntax,
         Binding->Protseq, Binding->NetworkAddr, Binding->Endpoint,
-        Binding->AuthInfo);
+        Binding->AuthInfo, Binding->QOS);
     if (NewConnection) {
       *Connection = NewConnection;
       return RPC_S_OK;
@@ -255,9 +264,13 @@ RPC_STATUS RPCRT4_OpenBinding(RpcBinding* Binding, RpcConnection** Connection,
   }
   
   /* create a new connection */
-  RPCRT4_CreateConnection(&NewConnection, Binding->server, Binding->Protseq,
-                          Binding->NetworkAddr, Binding->Endpoint, NULL,
-                          Binding->AuthInfo, Binding);
+  status = RPCRT4_CreateConnection(&NewConnection, Binding->server,
+                                   Binding->Protseq, Binding->NetworkAddr,
+                                   Binding->Endpoint, Binding->NetworkOptions,
+                                   Binding->AuthInfo, Binding->QOS, Binding);
+  if (status != RPC_S_OK)
+    return status;
+
   status = RPCRT4_OpenClientConnection(NewConnection);
   if (status != RPC_S_OK)
   {
@@ -867,9 +880,12 @@ RPC_STATUS RPC_ENTRY RpcBindingCopy(
   DestBinding->Protseq = RPCRT4_strndupA(SrcBinding->Protseq, -1);
   DestBinding->NetworkAddr = RPCRT4_strndupA(SrcBinding->NetworkAddr, -1);
   DestBinding->Endpoint = RPCRT4_strndupA(SrcBinding->Endpoint, -1);
+  DestBinding->NetworkOptions = RPCRT4_strdupW(SrcBinding->NetworkOptions);
 
   if (SrcBinding->AuthInfo) RpcAuthInfo_AddRef(SrcBinding->AuthInfo);
   DestBinding->AuthInfo = SrcBinding->AuthInfo;
+  if (SrcBinding->QOS) RpcQualityOfService_AddRef(SrcBinding->QOS);
+  DestBinding->QOS = SrcBinding->QOS;
 
   *DestinationBinding = DestBinding;
   return RPC_S_OK;
@@ -930,6 +946,7 @@ static RPC_STATUS RpcAuthInfo_Create(unsigned long AuthnLevel, unsigned long Aut
     if (!AuthInfo)
         return ERROR_OUTOFMEMORY;
 
+    AuthInfo->refs = 1;
     AuthInfo->AuthnLevel = AuthnLevel;
     AuthInfo->AuthnSvc = AuthnSvc;
     AuthInfo->cred = cred;
@@ -956,6 +973,146 @@ ULONG RpcAuthInfo_Release(RpcAuthInfo *AuthInfo)
     return refs;
 }
 
+static RPC_STATUS RpcQualityOfService_Create(const RPC_SECURITY_QOS *qos_src, BOOL unicode, RpcQualityOfService **qos_dst)
+{
+    RpcQualityOfService *qos = HeapAlloc(GetProcessHeap(), 0, sizeof(*qos));
+
+    if (!qos)
+        return RPC_S_OUT_OF_RESOURCES;
+
+    qos->refs = 1;
+    qos->qos = HeapAlloc(GetProcessHeap(), 0, sizeof(*qos->qos));
+    if (!qos->qos) goto error;
+    qos->qos->Version = qos_src->Version;
+    qos->qos->Capabilities = qos_src->Capabilities;
+    qos->qos->IdentityTracking = qos_src->IdentityTracking;
+    qos->qos->ImpersonationType = qos_src->ImpersonationType;
+    qos->qos->AdditionalSecurityInfoType = 0;
+
+    if (qos_src->Version >= 2)
+    {
+        const RPC_SECURITY_QOS_V2_W *qos_src2 = (const RPC_SECURITY_QOS_V2_W *)qos_src;
+        qos->qos->AdditionalSecurityInfoType = qos_src2->AdditionalSecurityInfoType;
+        if (qos_src2->AdditionalSecurityInfoType == RPC_C_AUTHN_INFO_TYPE_HTTP)
+        {
+            const RPC_HTTP_TRANSPORT_CREDENTIALS_W *http_credentials_src = qos_src2->u.HttpCredentials;
+            RPC_HTTP_TRANSPORT_CREDENTIALS_W *http_credentials_dst;
+
+            http_credentials_dst = HeapAlloc(GetProcessHeap(), 0, sizeof(*http_credentials_dst));
+            qos->qos->u.HttpCredentials = http_credentials_dst;
+            if (!http_credentials_dst) goto error;
+            http_credentials_dst->TransportCredentials = NULL;
+            http_credentials_dst->Flags = http_credentials_src->Flags;
+            http_credentials_dst->AuthenticationTarget = http_credentials_src->AuthenticationTarget;
+            http_credentials_dst->NumberOfAuthnSchemes = http_credentials_src->NumberOfAuthnSchemes;
+            http_credentials_dst->AuthnSchemes = NULL;
+            http_credentials_dst->ServerCertificateSubject = NULL;
+            if (http_credentials_src->TransportCredentials)
+            {
+                SEC_WINNT_AUTH_IDENTITY_W *cred_dst;
+                cred_dst = http_credentials_dst->TransportCredentials = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*cred_dst));
+                if (!cred_dst) goto error;
+                cred_dst->Flags = SEC_WINNT_AUTH_IDENTITY_UNICODE;
+                if (unicode)
+                {
+                    const SEC_WINNT_AUTH_IDENTITY_W *cred_src = http_credentials_src->TransportCredentials;
+                    cred_dst->UserLength = cred_src->UserLength;
+                    cred_dst->PasswordLength = cred_src->PasswordLength;
+                    cred_dst->DomainLength = cred_src->DomainLength;
+                    cred_dst->User = RPCRT4_strndupW(cred_src->User, cred_src->UserLength);
+                    cred_dst->Password = RPCRT4_strndupW(cred_src->Password, cred_src->PasswordLength);
+                    cred_dst->Domain = RPCRT4_strndupW(cred_src->Domain, cred_src->DomainLength);
+                }
+                else
+                {
+                    const SEC_WINNT_AUTH_IDENTITY_A *cred_src = (const SEC_WINNT_AUTH_IDENTITY_A *)http_credentials_src->TransportCredentials;
+                    cred_dst->UserLength = MultiByteToWideChar(CP_ACP, 0, (char *)cred_src->User, cred_src->UserLength, NULL, 0);
+                    cred_dst->DomainLength = MultiByteToWideChar(CP_ACP, 0, (char *)cred_src->Domain, cred_src->DomainLength, NULL, 0);
+                    cred_dst->PasswordLength = MultiByteToWideChar(CP_ACP, 0, (char *)cred_src->Password, cred_src->PasswordLength, NULL, 0);
+                    cred_dst->User = HeapAlloc(GetProcessHeap(), 0, cred_dst->UserLength * sizeof(WCHAR));
+                    cred_dst->Password = HeapAlloc(GetProcessHeap(), 0, cred_dst->PasswordLength * sizeof(WCHAR));
+                    cred_dst->Domain = HeapAlloc(GetProcessHeap(), 0, cred_dst->DomainLength * sizeof(WCHAR));
+                    if (!cred_dst || !cred_dst->Password || !cred_dst->Domain) goto error;
+                    MultiByteToWideChar(CP_ACP, 0, (char *)cred_src->User, cred_src->UserLength, cred_dst->User, cred_dst->UserLength);
+                    MultiByteToWideChar(CP_ACP, 0, (char *)cred_src->Domain, cred_src->DomainLength, cred_dst->Domain, cred_dst->DomainLength);
+                    MultiByteToWideChar(CP_ACP, 0, (char *)cred_src->Password, cred_src->PasswordLength, cred_dst->Password, cred_dst->PasswordLength);
+                }
+            }
+            if (http_credentials_src->NumberOfAuthnSchemes)
+            {
+                http_credentials_dst->AuthnSchemes = HeapAlloc(GetProcessHeap(), 0, http_credentials_src->NumberOfAuthnSchemes * sizeof(*http_credentials_dst->AuthnSchemes));
+                if (!http_credentials_dst->AuthnSchemes) goto error;
+                memcpy(http_credentials_dst->AuthnSchemes, http_credentials_src->AuthnSchemes, http_credentials_src->NumberOfAuthnSchemes * sizeof(*http_credentials_dst->AuthnSchemes));
+            }
+            if (http_credentials_src->ServerCertificateSubject)
+            {
+                if (unicode)
+                    http_credentials_dst->ServerCertificateSubject =
+                        RPCRT4_strndupW(http_credentials_src->ServerCertificateSubject,
+                                        strlenW(http_credentials_src->ServerCertificateSubject));
+                else
+                    http_credentials_dst->ServerCertificateSubject =
+                        RPCRT4_strdupAtoW((char *)http_credentials_src->ServerCertificateSubject);
+                if (!http_credentials_dst->ServerCertificateSubject) goto error;
+            }
+        }
+    }
+    *qos_dst = qos;
+    return RPC_S_OK;
+
+error:
+    if (qos->qos)
+    {
+        if (qos->qos->AdditionalSecurityInfoType == RPC_C_AUTHN_INFO_TYPE_HTTP &&
+            qos->qos->u.HttpCredentials)
+        {
+            if (qos->qos->u.HttpCredentials->TransportCredentials)
+            {
+                HeapFree(GetProcessHeap(), 0, qos->qos->u.HttpCredentials->TransportCredentials->User);
+                HeapFree(GetProcessHeap(), 0, qos->qos->u.HttpCredentials->TransportCredentials->Domain);
+                HeapFree(GetProcessHeap(), 0, qos->qos->u.HttpCredentials->TransportCredentials->Password);
+                HeapFree(GetProcessHeap(), 0, qos->qos->u.HttpCredentials->TransportCredentials);
+            }
+            HeapFree(GetProcessHeap(), 0, qos->qos->u.HttpCredentials->AuthnSchemes);
+            HeapFree(GetProcessHeap(), 0, qos->qos->u.HttpCredentials->ServerCertificateSubject);
+            HeapFree(GetProcessHeap(), 0, qos->qos->u.HttpCredentials);
+        }
+        HeapFree(GetProcessHeap(), 0, qos->qos);
+    }
+    HeapFree(GetProcessHeap(), 0, qos);
+    return RPC_S_OUT_OF_RESOURCES;
+}
+
+ULONG RpcQualityOfService_AddRef(RpcQualityOfService *qos)
+{
+    return InterlockedIncrement(&qos->refs);
+}
+
+ULONG RpcQualityOfService_Release(RpcQualityOfService *qos)
+{
+    ULONG refs = InterlockedDecrement(&qos->refs);
+
+    if (!refs)
+    {
+        if (qos->qos->AdditionalSecurityInfoType == RPC_C_AUTHN_INFO_TYPE_HTTP)
+        {
+            if (qos->qos->u.HttpCredentials->TransportCredentials)
+            {
+                HeapFree(GetProcessHeap(), 0, qos->qos->u.HttpCredentials->TransportCredentials->User);
+                HeapFree(GetProcessHeap(), 0, qos->qos->u.HttpCredentials->TransportCredentials->Domain);
+                HeapFree(GetProcessHeap(), 0, qos->qos->u.HttpCredentials->TransportCredentials->Password);
+                HeapFree(GetProcessHeap(), 0, qos->qos->u.HttpCredentials->TransportCredentials);
+            }
+            HeapFree(GetProcessHeap(), 0, qos->qos->u.HttpCredentials->AuthnSchemes);
+            HeapFree(GetProcessHeap(), 0, qos->qos->u.HttpCredentials->ServerCertificateSubject);
+            HeapFree(GetProcessHeap(), 0, qos->qos->u.HttpCredentials);
+        }
+        HeapFree(GetProcessHeap(), 0, qos->qos);
+        HeapFree(GetProcessHeap(), 0, qos);
+    }
+    return refs;
+}
+
 /***********************************************************************
  *             RpcRevertToSelf (RPCRT4.@)
  */
@@ -1045,9 +1202,53 @@ RpcBindingSetAuthInfoExA( RPC_BINDING_HANDLE Binding, RPC_CSTR ServerPrincName,
   TRACE("%p %s %lu %lu %p %lu %p\n", Binding, debugstr_a((const char*)ServerPrincName),
         AuthnLevel, AuthnSvc, AuthIdentity, AuthzSvr, SecurityQos);
 
-  if (AuthnLevel != RPC_C_AUTHN_LEVEL_CONNECT)
+  if (SecurityQos)
   {
-    FIXME("unsupported AuthnLevel %lu\n", AuthnLevel);
+      RPC_STATUS status;
+
+      TRACE("SecurityQos { Version=%ld, Capabilties=0x%lx, IdentityTracking=%ld, ImpersonationLevel=%ld",
+            SecurityQos->Version, SecurityQos->Capabilities, SecurityQos->IdentityTracking, SecurityQos->ImpersonationType);
+      if (SecurityQos->Version >= 2)
+      {
+          const RPC_SECURITY_QOS_V2_A *SecurityQos2 = (const RPC_SECURITY_QOS_V2_A *)SecurityQos;
+          TRACE(", AdditionalSecurityInfoType=%ld", SecurityQos2->AdditionalSecurityInfoType);
+          if (SecurityQos2->AdditionalSecurityInfoType == RPC_C_AUTHN_INFO_TYPE_HTTP)
+              TRACE(", { %p, 0x%lx, %ld, %ld, %p, %s }",
+                    SecurityQos2->u.HttpCredentials->TransportCredentials,
+                    SecurityQos2->u.HttpCredentials->Flags,
+                    SecurityQos2->u.HttpCredentials->AuthenticationTarget,
+                    SecurityQos2->u.HttpCredentials->NumberOfAuthnSchemes,
+                    SecurityQos2->u.HttpCredentials->AuthnSchemes,
+                    SecurityQos2->u.HttpCredentials->ServerCertificateSubject);
+      }
+      TRACE("}\n");
+      status = RpcQualityOfService_Create(SecurityQos, FALSE, &bind->QOS);
+      if (status != RPC_S_OK)
+          return status;
+  }
+  else
+  {
+      if (bind->QOS) RpcQualityOfService_Release(bind->QOS);
+      bind->QOS = NULL;
+  }
+
+  if (AuthnSvc == RPC_C_AUTHN_DEFAULT)
+    AuthnSvc = RPC_C_AUTHN_WINNT;
+
+  /* FIXME: the mapping should probably be retrieved using SSPI somehow */
+  if (AuthnLevel == RPC_C_AUTHN_LEVEL_DEFAULT)
+    AuthnLevel = RPC_C_AUTHN_LEVEL_NONE;
+
+  if ((AuthnLevel == RPC_C_AUTHN_LEVEL_NONE) || (AuthnSvc == RPC_C_AUTHN_NONE))
+  {
+    if (bind->AuthInfo) RpcAuthInfo_Release(bind->AuthInfo);
+    bind->AuthInfo = NULL;
+    return RPC_S_OK;
+  }
+
+  if (AuthnLevel > RPC_C_AUTHN_LEVEL_PKT_PRIVACY)
+  {
+    FIXME("unknown AuthnLevel %lu\n", AuthnLevel);
     return RPC_S_UNKNOWN_AUTHN_LEVEL;
   }
 
@@ -1057,9 +1258,6 @@ RpcBindingSetAuthInfoExA( RPC_BINDING_HANDLE Binding, RPC_CSTR ServerPrincName,
     return RPC_S_UNKNOWN_AUTHZ_SERVICE;
   }
 
-  if (SecurityQos)
-    FIXME("SecurityQos ignored\n");
-
   r = EnumerateSecurityPackagesA(&package_count, &packages);
   if (r != SEC_E_OK)
   {
@@ -1117,9 +1315,53 @@ RpcBindingSetAuthInfoExW( RPC_BINDING_HANDLE Binding, RPC_WSTR ServerPrincName,
   TRACE("%p %s %lu %lu %p %lu %p\n", Binding, debugstr_w((const WCHAR*)ServerPrincName),
         AuthnLevel, AuthnSvc, AuthIdentity, AuthzSvr, SecurityQos);
 
-  if (AuthnLevel != RPC_C_AUTHN_LEVEL_CONNECT)
+  if (SecurityQos)
   {
-    FIXME("unsupported AuthnLevel %lu\n", AuthnLevel);
+      RPC_STATUS status;
+
+      TRACE("SecurityQos { Version=%ld, Capabilties=0x%lx, IdentityTracking=%ld, ImpersonationLevel=%ld",
+            SecurityQos->Version, SecurityQos->Capabilities, SecurityQos->IdentityTracking, SecurityQos->ImpersonationType);
+      if (SecurityQos->Version >= 2)
+      {
+          const RPC_SECURITY_QOS_V2_W *SecurityQos2 = (const RPC_SECURITY_QOS_V2_W *)SecurityQos;
+          TRACE(", AdditionalSecurityInfoType=%ld", SecurityQos2->AdditionalSecurityInfoType);
+          if (SecurityQos2->AdditionalSecurityInfoType == RPC_C_AUTHN_INFO_TYPE_HTTP)
+              TRACE(", { %p, 0x%lx, %ld, %ld, %p, %s }",
+                    SecurityQos2->u.HttpCredentials->TransportCredentials,
+                    SecurityQos2->u.HttpCredentials->Flags,
+                    SecurityQos2->u.HttpCredentials->AuthenticationTarget,
+                    SecurityQos2->u.HttpCredentials->NumberOfAuthnSchemes,
+                    SecurityQos2->u.HttpCredentials->AuthnSchemes,
+                    debugstr_w(SecurityQos2->u.HttpCredentials->ServerCertificateSubject));
+      }
+      TRACE("}\n");
+      status = RpcQualityOfService_Create(SecurityQos, TRUE, &bind->QOS);
+      if (status != RPC_S_OK)
+          return status;
+  }
+  else
+  {
+      if (bind->QOS) RpcQualityOfService_Release(bind->QOS);
+      bind->QOS = NULL;
+  }
+
+  if (AuthnSvc == RPC_C_AUTHN_DEFAULT)
+    AuthnSvc = RPC_C_AUTHN_WINNT;
+
+  /* FIXME: the mapping should probably be retrieved using SSPI somehow */
+  if (AuthnLevel == RPC_C_AUTHN_LEVEL_DEFAULT)
+    AuthnLevel = RPC_C_AUTHN_LEVEL_NONE;
+
+  if ((AuthnLevel == RPC_C_AUTHN_LEVEL_NONE) || (AuthnSvc == RPC_C_AUTHN_NONE))
+  {
+    if (bind->AuthInfo) RpcAuthInfo_Release(bind->AuthInfo);
+    bind->AuthInfo = NULL;
+    return RPC_S_OK;
+  }
+
+  if (AuthnLevel > RPC_C_AUTHN_LEVEL_PKT_PRIVACY)
+  {
+    FIXME("unknown AuthnLevel %lu\n", AuthnLevel);
     return RPC_S_UNKNOWN_AUTHN_LEVEL;
   }
 
@@ -1129,9 +1371,6 @@ RpcBindingSetAuthInfoExW( RPC_BINDING_HANDLE Binding, RPC_WSTR ServerPrincName,
     return RPC_S_UNKNOWN_AUTHZ_SERVICE;
   }
 
-  if (SecurityQos)
-    FIXME("SecurityQos ignored\n");
-
   r = EnumerateSecurityPackagesW(&package_count, &packages);
   if (r != SEC_E_OK)
   {
