@@ -139,28 +139,6 @@ BOOL WINAPI SetupCopyOEMInfW(PCWSTR sourceinffile, PCWSTR sourcemedialoc,
   return FALSE;
 }
 
-/***********************************************************************
- *      SetupGetSourceInfoA  (SETUPAPI.@)
- */
-BOOL WINAPI SetupGetSourceInfoA(HINF InfHandle, UINT SourceId, UINT InfoDesired,
-                PSTR ReturnBuffer, DWORD ReturnBufferSize, LPDWORD RequiredSize)
-{
-  FIXME("(%p, %d, %d, %p, %d, %p): stub\n", InfHandle, SourceId, InfoDesired,
-        ReturnBuffer, ReturnBufferSize, RequiredSize);
-  return FALSE;
-}
-
-/***********************************************************************
- *      SetupGetSourceInfoW  (SETUPAPI.@)
- */
-BOOL WINAPI SetupGetSourceInfoW(HINF InfHandle, UINT SourceId, UINT InfoDesired,
-                PWSTR ReturnBuffer, DWORD ReturnBufferSize, LPDWORD RequiredSize)
-{
-  FIXME("(%p, %d, %d, %p, %d, %p): stub\n", InfHandle, SourceId, InfoDesired,
-        ReturnBuffer, ReturnBufferSize, RequiredSize);
-  return FALSE;
-}
-
 /***********************************************************************
  *		SetupInitializeFileLogW(SETUPAPI.@)
  */
@@ -222,3 +200,12 @@ BOOL WINAPI SetupOpenLog(BOOL Reserved)
     FIXME("(%d) stub\n", Reserved);
     return TRUE;
 }
+
+/***********************************************************************
+ *      SetupPromptReboot(SETUPAPI.@)
+ */
+INT WINAPI SetupPromptReboot( HSPFILEQ file_queue, HWND owner, BOOL scan_only )
+{
+    FIXME("%p, %p, %d\n", file_queue, owner, scan_only);
+    return 0;
+}
