@@ -46,13 +46,13 @@ typedef struct {
 
 static void dump_BINDINFO(BINDINFO *bi)
 {
-    static const char *BINDINFOF_str[] = {
+    static const char * const BINDINFOF_str[] = {
         "#0",
         "BINDINFOF_URLENCODESTGMEDDATA",
         "BINDINFOF_URLENCODEDEXTRAINFO"
     };
 
-    static const char *BINDVERB_str[] = {
+    static const char * const BINDVERB_str[] = {
         "BINDVERB_GET",
         "BINDVERB_POST",
         "BINDVERB_PUT",
