@@ -50,7 +50,6 @@ WINE_DEFAULT_DEBUG_CHANNEL(shell);
 extern HRESULT WINAPI IFSFolder_Constructor(IUnknown * pUnkOuter, REFIID riid, LPVOID * ppv);
 
 static const WCHAR sShell32[12] = {'S','H','E','L','L','3','2','.','D','L','L','\0'};
-static const WCHAR sOLE32[10] = {'O','L','E','3','2','.','D','L','L','\0'};
 
 /**************************************************************************
  * Default ClassFactory types
@@ -122,7 +121,7 @@ HRESULT WINAPI SHCoCreateInstance(
 {
 	DWORD	hres;
 	IID	iid;
-	CLSID * myclsid = (CLSID*)clsid;
+	const	CLSID * myclsid = clsid;
 	WCHAR	sKeyName[MAX_PATH];
 	const	WCHAR sCLSID[7] = {'C','L','S','I','D','\\','\0'};
 	WCHAR	sClassID[60];
