@@ -466,23 +466,25 @@ static HRESULT WINAPI statusclb_OnDataAvailable(IBindStatusCallback *iface, DWOR
 
     CHECK_EXPECT2(OnDataAvailable);
 
-#if 0 /* Uncomment after removing BindToStorage hack. */
+    if (0)
+    {
+    /* FIXME: Uncomment after removing BindToStorage hack. */
     ok(pformatetc != NULL, "pformatetx == NULL\n");
     if(pformatetc) {
         ok(pformatetc->cfFormat == 0xc02d, "clipformat=%x\n", pformatetc->cfFormat); 
         ok(pformatetc->ptd == NULL, "ptd = %p\n", pformatetc->ptd);
-        ok(pformatetc->dwAspect == 1, "dwAspect=%ld\n", pformatetc->dwAspect);
-        ok(pformatetc->lindex == -1, "lindex=%ld\n", pformatetc->lindex);
-        ok(pformatetc->tymed == TYMED_ISTREAM, "tymed=%ld\n", pformatetc->tymed);
+        ok(pformatetc->dwAspect == 1, "dwAspect=%u\n", pformatetc->dwAspect);
+        ok(pformatetc->lindex == -1, "lindex=%d\n", pformatetc->lindex);
+        ok(pformatetc->tymed == TYMED_ISTREAM, "tymed=%u\n", pformatetc->tymed);
     }
 
     ok(pstgmed != NULL, "stgmeg == NULL\n");
     if(pstgmed) {
-        ok(pstgmed->tymed == TYMED_ISTREAM, "tymed=%ld\n", pstgmed->tymed);
+        ok(pstgmed->tymed == TYMED_ISTREAM, "tymed=%u\n", pstgmed->tymed);
         ok(U(*pstgmed).pstm != NULL, "pstm == NULL\n");
         ok(pstgmed->pUnkForRelease != NULL, "pUnkForRelease == NULL\n");
     }
-#endif
+    }
 
     if(U(*pstgmed).pstm) {
         do hres = IStream_Read(U(*pstgmed).pstm, buf, 512, &readed);
@@ -602,7 +604,7 @@ static void test_CreateAsyncBindCtxEx(void)
         IBindCtx_Release(bctx);
 }
 
-static void test_BindToStorage(void)
+static void test_BindToStorage(int protocol, BOOL emul)
 {
     IMoniker *mon;
     HRESULT hres;
@@ -613,6 +615,9 @@ static void test_BindToStorage(void)
     IUnknown *unk = (IUnknown*)0x00ff00ff;
     IBinding *bind;
 
+    test_protocol = protocol;
+    emulate_protocol = emul;
+
     hres = CreateAsyncBindCtx(0, &bsc, NULL, &bctx);
     ok(SUCCEEDED(hres), "CreateAsyncBindCtx failed: %08x\n\n", hres);
     if(FAILED(hres))
@@ -777,33 +782,24 @@ START_TEST(url)
     test_CreateAsyncBindCtxEx();
 
     trace("http test...\n");
-    emulate_protocol = FALSE;
-    test_protocol = HTTP_TEST;
-    test_BindToStorage();
+    test_BindToStorage(HTTP_TEST, FALSE);
 
     trace("about test...\n");
-    test_protocol = ABOUT_TEST;
     CoInitialize(NULL);
-    test_BindToStorage();
+    test_BindToStorage(ABOUT_TEST, FALSE);
     CoUninitialize();
 
     trace("emulated about test...\n");
-    emulate_protocol = TRUE;
-    test_protocol = ABOUT_TEST;
-    test_BindToStorage();
+    test_BindToStorage(ABOUT_TEST, TRUE);
 
     trace("file test...\n");
     create_file();
-    emulate_protocol = FALSE;
-    test_protocol = FILE_TEST;
-    test_BindToStorage();
+    test_BindToStorage(FILE_TEST, FALSE);
     DeleteFileW(wszIndexHtml);
 
     trace("emulated file test...\n");
     set_file_url();
-    emulate_protocol = TRUE;
-    test_protocol = FILE_TEST;
-    test_BindToStorage();
+    test_BindToStorage(FILE_TEST, TRUE);
 
     test_BindToStorage_fail();
 }
