@@ -181,6 +181,8 @@ static const ClassFactory FtpProtocolCF =
     { &ClassFactoryVtbl, FtpProtocol_Construct};
 static const ClassFactory HttpProtocolCF =
     { &ClassFactoryVtbl, HttpProtocol_Construct};
+static const ClassFactory MkProtocolCF =
+    { &ClassFactoryVtbl, MkProtocol_Construct};
 static const ClassFactory SecurityManagerCF =
     { &ClassFactoryVtbl, SecManagerImpl_Construct};
 static const ClassFactory ZoneManagerCF =
@@ -196,12 +198,14 @@ struct object_creation_info
 static const WCHAR wszFile[] = {'f','i','l','e',0};
 static const WCHAR wszFtp[]  = {'f','t','p',0};
 static const WCHAR wszHttp[] = {'h','t','t','p',0};
+static const WCHAR wszMk[]   = {'m','k',0};
 
 static const struct object_creation_info object_creation[] =
 {
     { &CLSID_FileProtocol,            CLASSFACTORY(&FileProtocolCF),    wszFile },
     { &CLSID_FtpProtocol,             CLASSFACTORY(&FtpProtocolCF),     wszFtp  },
     { &CLSID_HttpProtocol,            CLASSFACTORY(&HttpProtocolCF),    wszHttp },
+    { &CLSID_MkProtocol,              CLASSFACTORY(&MkProtocolCF),      wszMk },
     { &CLSID_InternetSecurityManager, CLASSFACTORY(&SecurityManagerCF), NULL    },
     { &CLSID_InternetZoneManager,     CLASSFACTORY(&ZoneManagerCF),     NULL    }
 };
@@ -303,12 +307,6 @@ HRESULT WINAPI ObtainUserAgentString(DWORD dwOption, LPSTR pcszUAOut, DWORD *cbS
     return S_OK;
 }
 
-HRESULT WINAPI CoInternetCompareUrl(LPCWSTR pwzUrl1, LPCWSTR pwzUrl2, DWORD dwCompareFlags)
-{
-    TRACE("(%s,%s,%08x)\n", debugstr_w(pwzUrl1), debugstr_w(pwzUrl2), dwCompareFlags);
-    return UrlCompareW(pwzUrl1, pwzUrl2, dwCompareFlags)==0?S_OK:S_FALSE;
-}
-
 /**************************************************************************
  *                 IsValidURL (URLMON.@)
  * 
@@ -452,7 +450,7 @@ static BOOL image_bmp_filter(const BYTE const *b, DWORD size)
 {
     return size >= 14
         && b[0] == 0x42 && b[1] == 0x4d
-        && *(DWORD*)(b+6) == 0;
+        && *(const DWORD *)(b+6) == 0;
 }
 
 static BOOL video_avi_filter(const BYTE const *b, DWORD size)
@@ -469,6 +467,31 @@ static BOOL video_mpeg_filter(const BYTE const *b, DWORD size)
         && (b[3] == 0xb3 || b[3] == 0xba);
 }
 
+static BOOL application_pdf_filter(const BYTE const *b, DWORD size)
+{
+    return size > 4 && b[0] == 0x25 && b[1] == 0x50 && b[2] == 0x44 && b[3] == 0x46;
+}
+
+static BOOL application_xzip_filter(const BYTE const *b, DWORD size)
+{
+    return size > 2 && b[0] == 0x50 && b[1] == 0x4b;
+}
+
+static BOOL application_xgzip_filter(const BYTE const *b, DWORD size)
+{
+    return size > 2 && b[0] == 0x1f && b[1] == 0x8b;
+}
+
+static BOOL application_java_filter(const BYTE const *b, DWORD size)
+{
+    return size > 4 && b[0] == 0xca && b[1] == 0xfe && b[2] == 0xba && b[3] == 0xbe;
+}
+
+static BOOL application_xmsdownload(const BYTE const *b, DWORD size)
+{
+    return size > 2 && b[0] == 'M' && b[1] == 'Z';
+}
+
 static BOOL text_plain_filter(const BYTE const *b, DWORD size)
 {
     const BYTE *ptr;
@@ -529,6 +552,16 @@ HRESULT WINAPI FindMimeFromData(LPBC pBC, LPCWSTR pwzUrl, LPVOID pBuffer,
         static const WCHAR wszImageBmp[] = {'i','m','a','g','e','/','b','m','p',0};
         static const WCHAR wszVideoAvi[] = {'v','i','d','e','o','/','a','v','i',0};
         static const WCHAR wszVideoMpeg[] = {'v','i','d','e','o','/','m','p','e','g',0};
+        static const WCHAR wszAppPdf[] = {'a','p','p','l','i','c','a','t','i','o','n','/',
+            'p','d','f',0};
+        static const WCHAR wszAppXZip[] = {'a','p','p','l','i','c','a','t','i','o','n','/',
+            'x','-','z','i','p','-','c','o','m','p','r','e','s','s','e','d',0};
+        static const WCHAR wszAppXGzip[] = {'a','p','p','l','i','c','a','t','i','o','n','/',
+            'x','-','g','z','i','p','-','c','o','m','p','r','e','s','s','e','d',0};
+        static const WCHAR wszAppJava[] = {'a','p','p','l','i','c','a','t','i','o','n','/',
+            'j','a','v','a',0};
+        static const WCHAR wszAppXMSDownload[] = {'a','p','p','l','i','c','a','t','i','o','n','/',
+            'x','-','m','s','d','o','w','n','l','o','a','d',0};
         static const WCHAR wszTextPlain[] = {'t','e','x','t','/','p','l','a','i','n','\0'};
         static const WCHAR wszAppOctetStream[] = {'a','p','p','l','i','c','a','t','i','o','n','/',
             'o','c','t','e','t','-','s','t','r','e','a','m','\0'};
@@ -545,6 +578,11 @@ HRESULT WINAPI FindMimeFromData(LPBC pBC, LPCWSTR pwzUrl, LPVOID pBuffer,
             {wszImageBmp,       image_bmp_filter},
             {wszVideoAvi,       video_avi_filter},
             {wszVideoMpeg,      video_mpeg_filter},
+            {wszAppPdf,         application_pdf_filter},
+            {wszAppXZip,        application_xzip_filter},
+            {wszAppXGzip,       application_xgzip_filter},
+            {wszAppJava,        application_java_filter},
+            {wszAppXMSDownload, application_xmsdownload},
             {wszTextPlain,      text_plain_filter},
             {wszAppOctetStream, application_octet_stream_filter}
         };
