@@ -267,7 +267,7 @@ typedef enum _WINED3DFORMAT {
     WINED3DFMT_DXT3                 =  MAKEFOURCC('D', 'X', 'T', '3'),
     WINED3DFMT_DXT4                 =  MAKEFOURCC('D', 'X', 'T', '4'),
     WINED3DFMT_DXT5                 =  MAKEFOURCC('D', 'X', 'T', '5'),
-    WINED3DFMT_MULTI2_ARGB          =  MAKEFOURCC('M', 'E', 'T', '1'),
+    WINED3DFMT_MULTI2_ARGB8         =  MAKEFOURCC('M', 'E', 'T', '1'),
     WINED3DFMT_G8R8_G8B8            =  MAKEFOURCC('G', 'R', 'G', 'B'),
     WINED3DFMT_R8G8_B8G8            =  MAKEFOURCC('R', 'G', 'B', 'G'),
 
@@ -286,7 +286,7 @@ typedef enum _WINED3DFORMAT {
     WINED3DFMT_INDEX16              = 101,
     WINED3DFMT_INDEX32              = 102,
     WINED3DFMT_Q16W16V16U16         = 110,
-    /* Flaoting point formats */
+    /* Floating point formats */
     WINED3DFMT_R16F                 = 111,
     WINED3DFMT_G16R16F              = 112,
     WINED3DFMT_A16B16G16R16F        = 113,
@@ -362,6 +362,40 @@ typedef enum _WINED3DRENDERSTATETYPE {
     WINED3DRS_STENCILMASK               =  58,
     WINED3DRS_STENCILWRITEMASK          =  59,
     WINED3DRS_TEXTUREFACTOR             =  60,
+
+    WINED3DRS_STIPPLEPATTERN00          = 64,
+    WINED3DRS_STIPPLEPATTERN01          = 65,
+    WINED3DRS_STIPPLEPATTERN02          = 66,
+    WINED3DRS_STIPPLEPATTERN03          = 67,
+    WINED3DRS_STIPPLEPATTERN04          = 68,
+    WINED3DRS_STIPPLEPATTERN05          = 69,
+    WINED3DRS_STIPPLEPATTERN06          = 70,
+    WINED3DRS_STIPPLEPATTERN07          = 71,
+    WINED3DRS_STIPPLEPATTERN08          = 72,
+    WINED3DRS_STIPPLEPATTERN09          = 73,
+    WINED3DRS_STIPPLEPATTERN10          = 74,
+    WINED3DRS_STIPPLEPATTERN11          = 75,
+    WINED3DRS_STIPPLEPATTERN12          = 76,
+    WINED3DRS_STIPPLEPATTERN13          = 77,
+    WINED3DRS_STIPPLEPATTERN14          = 78,
+    WINED3DRS_STIPPLEPATTERN15          = 79,
+    WINED3DRS_STIPPLEPATTERN16          = 80,
+    WINED3DRS_STIPPLEPATTERN17          = 81,
+    WINED3DRS_STIPPLEPATTERN18          = 82,
+    WINED3DRS_STIPPLEPATTERN19          = 83,
+    WINED3DRS_STIPPLEPATTERN20          = 84,
+    WINED3DRS_STIPPLEPATTERN21          = 85,
+    WINED3DRS_STIPPLEPATTERN22          = 86,
+    WINED3DRS_STIPPLEPATTERN23          = 87,
+    WINED3DRS_STIPPLEPATTERN24          = 88,
+    WINED3DRS_STIPPLEPATTERN25          = 89,
+    WINED3DRS_STIPPLEPATTERN26          = 90,
+    WINED3DRS_STIPPLEPATTERN27          = 91,
+    WINED3DRS_STIPPLEPATTERN28          = 92,
+    WINED3DRS_STIPPLEPATTERN29          = 93,
+    WINED3DRS_STIPPLEPATTERN30          = 94,
+    WINED3DRS_STIPPLEPATTERN31          = 95,
+
     WINED3DRS_WRAP0                     = 128,
     WINED3DRS_WRAP1                     = 129,
     WINED3DRS_WRAP2                     = 130,
@@ -1218,6 +1252,7 @@ typedef struct WineDirect3DStridedData {
     DWORD     dwStride;      /* Stride between occurances of this data */
     DWORD     dwType;        /* Type (as in D3DVSDT_TYPE)              */
     int       VBO;           /* Vertex buffer object this data is in   */
+    UINT      streamNo;      /* D3D stream number                      */
 } WineDirect3DStridedData;
 
 typedef struct WineDirect3DVertexStridedData {
