@@ -126,6 +126,8 @@ char string[8], outpath[MAX_PATH], inpath[MAX_PATH], *infile;
   }
 
   GetFullPathName (param2, sizeof(outpath), outpath, NULL);
+  if (outpath[strlen(outpath) - 1] == '\\')
+      outpath[strlen(outpath) - 1] = '\0';
   hff = FindFirstFile (outpath, &fd);
   if (hff != INVALID_HANDLE_VALUE) {
     if (fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) {
@@ -433,7 +435,7 @@ char string[MAX_PATH];
   if (context != NULL) {
     SetFilePointer (context -> h, 0, NULL, FILE_BEGIN);
     while (WCMD_fgets (string, sizeof(string), context -> h)) {
-      if ((string[0] == ':') && (strcmp (&string[1], param1) == 0)) return;
+      if ((string[0] == ':') && (lstrcmpi (&string[1], param1) == 0)) return;
     }
     WCMD_output ("Target to GOTO not found\n");
   }
@@ -462,7 +464,6 @@ char condition[MAX_PATH], *command, *s;
   }
   if (!lstrcmpi (condition, "errorlevel")) {
     if (errorlevel >= atoi(WCMD_parameter (p, 1+negate, NULL))) test = 1;
-    return;
     WCMD_parameter (p, 2+negate, &command);
   }
   else if (!lstrcmpi (condition, "exist")) {
