@@ -284,6 +284,7 @@ struct thread *create_process( int fd, struct thread *parent_thread, int inherit
     process->winstation      = 0;
     process->desktop         = 0;
     process->token           = token_create_admin();
+    process->trace_data      = 0;
     list_init( &process->thread_list );
     list_init( &process->locks );
     list_init( &process->classes );
@@ -343,6 +344,7 @@ data_size_t init_process( struct thread *thread )
     struct process *process = thread->process;
     struct startup_info *info = process->startup_info;
 
+    init_process_tracing( process );
     if (!info) return 0;
     return info->data_size;
 }
@@ -490,7 +492,7 @@ static struct process_dll *process_load_dll( struct process *process, struct fil
             free( dll );
             return NULL;
         }
-        if (file) dll->file = (struct file *)grab_object( file );
+        if (file) dll->file = grab_file_unless_removable( file );
         list_add_tail( &process->dlls, &dll->entry );
     }
     return dll;
@@ -517,7 +519,7 @@ static void terminate_process( struct process *process, struct thread *skip, int
 {
     struct list *ptr;
 
-    if (skip)  /* move it to the end of the list */
+    if (skip && skip->process == process)  /* move it to the end of the list */
     {
         assert( skip->state != TERMINATED );
         list_remove( &skip->proc_entry );
@@ -599,6 +601,7 @@ static void process_killed( struct process *process )
     destroy_process_classes( process );
     remove_process_locks( process );
     set_process_startup_state( process, STARTUP_ABORTED );
+    finish_process_tracing( process );
     start_sigkill_timer( process );
     wake_up( &process->obj, 0 );
 }
