@@ -129,6 +129,7 @@ DECL_HANDLER(load_dll);
 DECL_HANDLER(unload_dll);
 DECL_HANDLER(queue_apc);
 DECL_HANDLER(get_apc);
+DECL_HANDLER(get_apc_result);
 DECL_HANDLER(close_handle);
 DECL_HANDLER(set_handle_info);
 DECL_HANDLER(dup_handle);
@@ -323,6 +324,7 @@ DECL_HANDLER(open_directory);
 DECL_HANDLER(create_symlink);
 DECL_HANDLER(open_symlink);
 DECL_HANDLER(query_symlink);
+DECL_HANDLER(get_object_info);
 
 #ifdef WANT_REQUEST_HANDLERS
 
@@ -348,6 +350,7 @@ static const req_handler req_handlers[REQ_NB_REQUESTS] =
     (req_handler)req_unload_dll,
     (req_handler)req_queue_apc,
     (req_handler)req_get_apc,
+    (req_handler)req_get_apc_result,
     (req_handler)req_close_handle,
     (req_handler)req_set_handle_info,
     (req_handler)req_dup_handle,
@@ -542,6 +545,7 @@ static const req_handler req_handlers[REQ_NB_REQUESTS] =
     (req_handler)req_create_symlink,
     (req_handler)req_open_symlink,
     (req_handler)req_query_symlink,
+    (req_handler)req_get_object_info,
 };
 #endif  /* WANT_REQUEST_HANDLERS */
 
