@@ -424,6 +424,7 @@ static struct token *create_token( unsigned primary, const SID *user,
         list_init( &token->privileges );
         list_init( &token->groups );
         token->primary = primary;
+        token->default_dacl = NULL;
 
         /* copy user */
         token->user = memdup( user, FIELD_OFFSET(SID, SubAuthority[user->SubAuthorityCount]) );
@@ -477,8 +478,6 @@ static struct token *create_token( unsigned primary, const SID *user,
                 return NULL;
             }
         }
-        else
-            token->default_dacl = NULL;
 
         token->source = source;
     }
@@ -515,7 +514,7 @@ static ACL *create_default_dacl( const SID *user )
     memcpy( sid, &local_system_sid, sizeof(local_system_sid) );
 
     /* GENERIC_ALL for specified user */
-    aaa = (ACCESS_ALLOWED_ACE *)((const char *)aaa + aaa->Header.AceSize);
+    aaa = (ACCESS_ALLOWED_ACE *)((char *)aaa + aaa->Header.AceSize);
     aaa->Header.AceType = ACCESS_ALLOWED_ACE_TYPE;
     aaa->Header.AceFlags = 0;
     aaa->Header.AceSize = (sizeof(ACCESS_ALLOWED_ACE) - sizeof(DWORD)) +
@@ -542,7 +541,8 @@ struct token *token_create_admin( void )
     static const unsigned int alias_users_subauth[] = { SECURITY_BUILTIN_DOMAIN_RID, DOMAIN_ALIAS_RID_USERS };
     PSID alias_admins_sid;
     PSID alias_users_sid;
-    ACL *default_dacl = create_default_dacl( &local_system_sid );
+    /* note: should be the owner specified in the token */
+    ACL *default_dacl = create_default_dacl( &interactive_sid );
 
     alias_admins_sid = security_sid_alloc( &nt_authority, sizeof(alias_admins_subauth)/sizeof(alias_admins_subauth[0]),
                                            alias_admins_subauth );
@@ -706,11 +706,12 @@ static int token_sid_present( struct token *token, const SID *sid, int deny )
     return FALSE;
 }
 
-/* checks access to a security descriptor. sd must have been validated by caller.
- * it returns STATUS_SUCCESS if access was granted to the object, or an error
- * status code if not, giving the reason. errors not relating to giving access
- * to the object are returned in the status parameter. granted_access and
- * status always have a valid value stored in them on return. */
+/* Checks access to a security descriptor. 'sd' must have been validated by
+ * caller. It returns STATUS_SUCCESS if call succeeded or an error indicating
+ * the reason. 'status' parameter will indicate if access is granted or denied.
+ *
+ * If both returned value and 'status' are STATUS_SUCCESS then access is granted.
+ */
 static unsigned int token_access_check( struct token *token,
                                  const struct security_descriptor *sd,
                                  unsigned int desired_access,
@@ -728,16 +729,14 @@ static unsigned int token_access_check( struct token *token,
     const ACE_HEADER *ace;
     const SID *owner;
 
-    /* assume success, but no access rights */
-    *status = STATUS_SUCCESS;
+    /* assume no access rights */
     *granted_access = 0;
 
     /* fail if desired_access contains generic rights */
     if (desired_access & (GENERIC_READ|GENERIC_WRITE|GENERIC_EXECUTE|GENERIC_ALL))
     {
         *priv_count = 0;
-        *status = STATUS_GENERIC_NOT_MAPPED;
-        return STATUS_ACCESS_DENIED;
+        return STATUS_GENERIC_NOT_MAPPED;
     }
 
     dacl = sd_get_dacl( sd, &dacl_present );
@@ -745,8 +744,7 @@ static unsigned int token_access_check( struct token *token,
     if (!owner || !sd_get_group( sd ))
     {
         *priv_count = 0;
-        *status = STATUS_INVALID_SECURITY_DESCR;
-        return STATUS_ACCESS_DENIED;
+        return STATUS_INVALID_SECURITY_DESCR;
     }
 
     /* 1: Grant desired access if the object is unprotected */
@@ -754,12 +752,13 @@ static unsigned int token_access_check( struct token *token,
     {
         *priv_count = 0;
         *granted_access = desired_access;
-        return STATUS_SUCCESS;
+        return *status = STATUS_SUCCESS;
     }
     if (!dacl)
     {
         *priv_count = 0;
-        return STATUS_ACCESS_DENIED;
+        *status = STATUS_ACCESS_DENIED;
+        return STATUS_SUCCESS;
     }
 
     /* 2: Check if caller wants access to system security part. Note: access
@@ -788,13 +787,14 @@ static unsigned int token_access_check( struct token *token,
             if (desired_access == current_access)
             {
                 *granted_access = current_access;
-                return STATUS_SUCCESS;
+                return *status = STATUS_SUCCESS;
             }
         }
         else
         {
             *priv_count = 0;
-            return STATUS_PRIVILEGE_NOT_HELD;
+            *status = STATUS_PRIVILEGE_NOT_HELD;
+            return STATUS_SUCCESS;
         }
     }
     else if (priv_count) *priv_count = 0;
@@ -809,7 +809,7 @@ static unsigned int token_access_check( struct token *token,
         if (desired_access == current_access)
         {
             *granted_access = current_access;
-            return STATUS_SUCCESS;
+            return *status = STATUS_SUCCESS;
         }
     }
 
@@ -834,11 +834,7 @@ static unsigned int token_access_check( struct token *token,
                 else
                 {
                     denied_access |= (access & ~current_access);
-                    if (desired_access & access)
-                    {
-                        *granted_access = 0;
-                        return STATUS_SUCCESS;
-                    }
+                    if (desired_access & access) goto done;
                 }
             }
             break;
@@ -865,24 +861,17 @@ static unsigned int token_access_check( struct token *token,
         ace = ace_next( ace );
     }
 
+done:
     if (desired_access & MAXIMUM_ALLOWED)
-    {
         *granted_access = current_access & ~denied_access;
-        if (*granted_access)
-            return STATUS_SUCCESS;
-        else
-            return STATUS_ACCESS_DENIED;
-    }
     else
-    {
         if ((current_access & desired_access) == desired_access)
-        {
             *granted_access = current_access & desired_access;
-            return STATUS_SUCCESS;
-        }
         else
-            return STATUS_ACCESS_DENIED;
-    }
+            *granted_access = 0;
+
+    *status = *granted_access ? STATUS_SUCCESS : STATUS_ACCESS_DENIED;
+    return STATUS_SUCCESS;
 }
 
 const ACL *token_get_default_dacl( struct token *token )
@@ -1108,9 +1097,9 @@ DECL_HANDLER(access_check)
         mapping.GenericExecute = req->mapping_execute;
         mapping.GenericAll = req->mapping_all;
 
-        reply->access_status = token_access_check(
+        status = token_access_check(
             token, sd, req->desired_access, &priv, &priv_count, &mapping,
-            &reply->access_granted, &status );
+            &reply->access_granted, &reply->access_status );
 
         reply->privileges_len = priv_count*sizeof(LUID_AND_ATTRIBUTES);
 
@@ -1120,9 +1109,7 @@ DECL_HANDLER(access_check)
             memcpy( privs, &priv, sizeof(priv) );
         }
 
-        if (status != STATUS_SUCCESS)
-            set_error( status );
-
+        set_error( status );
         release_object( token );
     }
 }
