@@ -24,6 +24,7 @@
 #include <stdarg.h>
 #include "guiddef.h"
 #include "wine/rpcfc.h"
+#include "wine/list.h"
 
 #ifndef UUID_DEFINED
 #define UUID_DEFINED
@@ -45,23 +46,12 @@ typedef struct _importlib_t importlib_t;
 typedef struct _importinfo_t importinfo_t;
 typedef struct _typelib_t typelib_t;
 
-#define DECL_LINK(type) \
-  type *l_next; \
-  type *l_prev
-
-#define LINK(x,y) do { x->l_next = y; x->l_prev = NULL; if (y) y->l_prev = x; } while (0)
-
-#define INIT_LINK(x) do { x->l_next = NULL; x->l_prev = NULL; } while (0)
-#define NEXT_LINK(x) ((x)->l_next)
-#define PREV_LINK(x) ((x)->l_prev)
-
-#define END_OF_LIST(list)       \
-  do {                          \
-    if (list) {                 \
-      while (NEXT_LINK(list))   \
-        list = NEXT_LINK(list); \
-    }                           \
-  } while(0)
+typedef struct list attr_list_t;
+typedef struct list func_list_t;
+typedef struct list expr_list_t;
+typedef struct list var_list_t;
+typedef struct list ifref_list_t;
+typedef struct list array_dims_t;
 
 enum attr_type
 {
@@ -171,7 +161,7 @@ enum type_kind
     TKIND_UNION,
     TKIND_MAX
 };
-   
+
 struct _attr_t {
   enum attr_type type;
   union {
@@ -179,7 +169,7 @@ struct _attr_t {
     void *pval;
   } u;
   /* parser-internal */
-  DECL_LINK(attr_t);
+  struct list entry;
 };
 
 struct _expr_t {
@@ -195,7 +185,7 @@ struct _expr_t {
   int is_const;
   long cval;
   /* parser-internal */
-  DECL_LINK(expr_t);
+  struct list entry;
 };
 
 struct _type_t {
@@ -203,16 +193,14 @@ struct _type_t {
   enum type_kind kind;
   unsigned char type;
   struct _type_t *ref;
-  const attr_t *attrs;
-  func_t *funcs;                  /* interfaces and modules */
-  var_t *fields;                  /* interfaces, structures and enumerations */
-  ifref_t *ifaces;                /* coclasses */
+  const attr_list_t *attrs;
+  func_list_t *funcs;             /* interfaces and modules */
+  var_list_t *fields;             /* interfaces, structures and enumerations */
+  ifref_list_t *ifaces;           /* coclasses */
   type_t *orig;                   /* dup'd types */
   int ignore, is_const, sign;
   int defined, written, user_types_registered;
   int typelib_idx;
-  /* parser-internal */
-  DECL_LINK(type_t);
 };
 
 struct _typeref_t {
@@ -224,37 +212,37 @@ struct _typeref_t {
 struct _var_t {
   char *name;
   int ptr_level;
-  expr_t *array;
+  array_dims_t *array;
   type_t *type;
-  var_t *args;  /* for function pointers */
+  var_list_t *args;  /* for function pointers */
   const char *tname;
-  attr_t *attrs;
+  attr_list_t *attrs;
   expr_t *eval;
 
   /* parser-internal */
-  DECL_LINK(var_t);
+  struct list entry;
 };
 
 struct _func_t {
   var_t *def;
-  var_t *args;
+  var_list_t *args;
   int ignore, idx;
 
   /* parser-internal */
-  DECL_LINK(func_t);
+  struct list entry;
 };
 
 struct _ifref_t {
   type_t *iface;
-  attr_t *attrs;
+  attr_list_t *attrs;
 
   /* parser-internal */
-  DECL_LINK(ifref_t);
+  struct list entry;
 };
 
 struct _typelib_entry_t {
     type_t *type;
-    DECL_LINK(typelib_entry_t);
+    struct list entry;
 };
 
 struct _importinfo_t {
@@ -279,15 +267,15 @@ struct _importlib_t {
 
     int allocated;
 
-    DECL_LINK(importlib_t);
+    struct list entry;
 };
 
 struct _typelib_t {
     char *name;
     char *filename;
-    attr_t *attrs;
-    typelib_entry_t *entry;
-    importlib_t *importlibs;
+    attr_list_t *attrs;
+    struct list entries;
+    struct list importlibs;
 };
 
 void init_types(void);
@@ -296,7 +284,7 @@ type_t *duptype(type_t *t, int dupname);
 type_t *alias(type_t *t, const char *name);
 
 int is_ptr(const type_t *t);
-int is_var_ptr(var_t *v);
-int cant_be_null(var_t *v);
+int is_var_ptr(const var_t *v);
+int cant_be_null(const var_t *v);
 
 #endif
